import { MatcherException } from '../../exceptions/matcher.exception';
import { IRequestGeography } from '../interfaces/geography-request.interface';
import { PointType } from '../types/geography.enum';
import { Point } from '../types/point.type';
import { Route } from './route';
import { find } from 'geo-tz';
import { Waypoint } from '../types/waypoint';

export class Geography {
  _geographyRequest: IRequestGeography;
  waypoints: Array<Waypoint>;
  originType: PointType;
  destinationType: PointType;
  timezones: Array<string>;
  driverRoute: Route;
  passengerRoute: Route;

  constructor(geographyRequest: IRequestGeography, defaultTimezone: string) {
    this._geographyRequest = geographyRequest;
    this.waypoints = [];
    this.originType = PointType.OTHER;
    this.destinationType = PointType.OTHER;
    this.timezones = [defaultTimezone];
  }

  init() {
    this._validateWaypoints();
    this._setTimezones();
  }

  _validateWaypoints() {
    if (this._geographyRequest.waypoints.length < 2) {
      throw new MatcherException(3, 'At least 2 waypoints are required');
    }
    this._geographyRequest.waypoints.map((point) => {
      if (!this._isValidPoint(point)) {
        throw new MatcherException(
          3,
          `Waypoint { Lon: ${point.lon}, Lat: ${point.lat} } is not valid`,
        );
      }
      this.waypoints.push({
        point,
        actors: [],
      });
    });
  }

  _setTimezones() {
    this.timezones = find(
      this._geographyRequest.waypoints[0].lat,
      this._geographyRequest.waypoints[0].lon,
    );
  }

  _isValidPoint = (point: Point): boolean =>
    this._isValidLongitude(point.lon) && this._isValidLatitude(point.lat);

  _isValidLongitude = (longitude: number): boolean =>
    longitude >= -180 && longitude <= 180;

  _isValidLatitude = (latitude: number): boolean =>
    latitude >= -90 && latitude <= 90;
}
